@@ -4,7 +4,7 @@
 # SOAP::Lite is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
 #
-# $Id: HTTP.pm 386 2011-08-18 19:48:31Z kutterma $
+# $Id$
 #
 # ======================================================================
 
@@ -12,7 +12,7 @@ package SOAP::Transport::HTTP;
 
 use strict;
 
-our $VERSION = 0.714;
+our $VERSION = 0.716;
 
 use SOAP::Lite;
 use SOAP::Packager;
@@ -44,7 +44,8 @@ sub patch {
     }
     {
 
-        package LWP::Protocol;
+        package
+            LWP::Protocol;
         local $^W = 0;
         my $collect = \&collect;    # store original
         *collect = sub {
@@ -131,6 +132,8 @@ sub send_receive {
     my ( $context, $envelope, $endpoint, $action, $encoding, $parts ) =
       @parameters{qw(context envelope endpoint action encoding parts)};
 
+    $encoding ||= 'UTF-8';
+
     $endpoint ||= $self->endpoint;
 
     my $method = 'POST';
@@ -170,7 +173,6 @@ sub send_receive {
           && $self->options->{is_compress}
           && ( $self->options->{compress_threshold} || 0 ) < length $envelope;
 
-        $envelope = Compress::Zlib::memGzip($envelope) if $compressed;
 
         my $original_encoding = $http_request->content_encoding;
 
@@ -202,10 +204,15 @@ sub send_receive {
             }
             else {
                 require Encode;
-                $envelope = Encode::encode('UTF-8', $envelope);
+                $envelope = Encode::encode($encoding, $envelope);
             }
             #  if !$SOAP::Constants::DO_NOT_USE_LWP_LENGTH_HACK
             #      && length($envelope) != $bytelength;
+
+            # compress after encoding
+            # doing it before breaks the compressed content (#74577)
+            $envelope = Compress::Zlib::memGzip($envelope) if $compressed;
+
             $http_request->content($envelope);
             $http_request->protocol('HTTP/1.1');
 
@@ -297,6 +304,7 @@ sub send_receive {
     $self->message( $self->http_response->message );
     $self->is_success( $self->http_response->is_success );
     $self->status( $self->http_response->status_line );
+    return if ($self->http_response->is_success == 0);
 
     # Pull out any cookies from the response headers
     $self->{'_cookie_jar'}->extract_cookies( $self->http_response )
@@ -691,7 +699,13 @@ sub handle {
         while ( my $r = $c->get_request ) {
             $self->request($r);
             $self->SUPER::handle;
-            $c->send_response( $self->response );
+            eval {
+                local $SIG{PIPE} = sub {die "SIGPIPE"};
+                $c->send_response( $self->response );
+            };
+            if ($@ && $@ !~ /^SIGPIPE/) {
+                die $@;
+            }
         }
 
 # replaced ->close, thanks to Sean Meisner <Sean.Meisner@VerizonWireless.com>
@@ -806,7 +820,7 @@ sub handler {
     $self->request(
         HTTP::Request->new(
             $r->method() => $r->uri,
-            HTTP::Headers->new( $r->headers_in ),
+            HTTP::Headers->new( %{ $r->headers_in } ),
             $content
         ) );
     $self->SUPER::handle;
