@@ -10,6 +10,6 @@ BEGIN {
 
 use Test::Harness;
 
-@ARGV = <*.t> unless @ARGV;
+@ARGV = sort <*.t> unless @ARGV;
 
-runtests(sort @ARGV);
\ No newline at end of file
+runtests(@ARGV);
\ No newline at end of file
